#include <cstdio>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <unistd.h>

#include "papu.hh"
#include "clock.h"

bisqpapu::bisqpapu() : Rate(48000)
{
	fd = 1;
	Rate = 44100;
	/*
	fd = open("/dev/dsp", O_WRONLY);
	if(fd >= 0)
	{
		int Bits = 16, Stereo = 0;
		ioctl(fd, SNDCTL_DSP_SAMPLESIZE, &Bits);
		ioctl(fd, SNDCTL_DSP_STEREO,     &Stereo);
		if(ioctl(fd, SNDCTL_DSP_SPEED, &Rate) < 0)
		{
			Rate = 44100;
			if(ioctl(fd, SNDCTL_DSP_SPEED, &Rate) < 0)
			{
				Rate = 22050;
				if(ioctl(fd, SNDCTL_DSP_SPEED, &Rate) < 0)
				{
					Rate = 11025;
					if(ioctl(fd, SNDCTL_DSP_SPEED, &Rate) < 0)
					{
						Rate = 8000;
						ioctl(fd, SNDCTL_DSP_SPEED, &Rate);
					}
				}
			}
		}
	}
	else
	{
		int pip[2], p;
		pipe(pip);
		p = fork();
		if(!p)
		{
			close(pip[1]); close(1); close(2); dup2(pip[0], 0); close(pip[0]);
			Rate = 44100;
			char ratebuf[6];
			sprintf(ratebuf, "%u", Rate);
			execlp("esdcat", "esdcat", "-m", "-r", ratebuf, NULL);
		}
		close(pip[0]);
		fd = pip[1];
	}
	*/
	
	counter = 0;
	speed = double(Clock->CPUclockDivider * Rate) / Clock->ClockHz;
}

bisqpapu::~bisqpapu()
{
}

void bisqpapu::mixbytes(unsigned count)
{
	short c = (chan1.outputvalue + chan2.outputvalue
    	      -chan3.outputvalue - chan4.outputvalue) * 256;
	do {
		std::write(fd, &c, 2);
	} while(--count);
}

bool bisqpapu::write(const fWORD addr, fBYTE value)
{
	switch(addr)
	{
		case 0x4000 ... 0x4003: fprintf(stderr, "\33[1m1\33[m");chan1.write(addr&3, value); return true;
		case 0x4004 ... 0x4007: fprintf(stderr, "\33[1m2\33[m");chan2.write(addr&3, value); return true;
		case 0x4008 ... 0x400B: fprintf(stderr, "\33[1m3\33[m");chan3.write(addr&3, value); return true;
		case 0x400C ... 0x400F: fprintf(stderr, "\33[1m4\33[m");chan4.write(addr&3, value); return true;
		case 0x4010 ... 0x4013: fprintf(stderr, "\33[1m5\33[m");chan5.write(addr&3, value); return true;
		case 0x4015:
			if(value & 0x01)chan1.enable();else chan1.disable();
			if(value & 0x02)chan2.enable();else chan2.disable();
			if(value & 0x04)chan3.enable();else chan3.disable();
			if(value & 0x08)chan4.enable();else chan4.disable();
			if(value & 0x10)chan5.enable();else chan5.disable();
			// &32, &64 and &128 ignored.
			return true;
	}
	return false;
}
bool bisqpapu::read(const fWORD addr, BYTE &value)
{
	switch(addr)
	{
		case 0x4000 ... 0x4013:
			// Complete. These are write-only.
			value = 0x40;
			return true;
		case 0x4015:
			if(!chan1.lengthunit_done())value |= 0x01;
			if(!chan2.lengthunit_done())value |= 0x02;
			if(!chan3.lengthunit_done())value |= 0x04;
			if(!chan4.lengthunit_done())value |= 0x08;
			// Bit 4: DMC status. TODO.
			// Bit 7: IRQ status of DMC. TODO.
			return true;
	}
	return false;
}

InterruptType bisqpapu::tick()
{
	// This function runs Clock->ClockHz / Clock->CPUclockDivider
	// times per second. For NTSC, it is 1.7897725 MHz.
	
	for(unsigned a=0; a<1; ++a)
	{
        chan1.tick();
        chan2.tick();
        chan3.tick();
        chan4.tick();
        //chan5.tick();
        
        counter += speed;
        register unsigned bytecount = (unsigned)counter;
        if(bytecount)
        {
        	mixbytes(bytecount);
        	counter -= bytecount;
        }
    }
	
	return INT_NONE;
}
