#include "ppu.hh"
#include "clock.h"

void bisqppu::writemem(fWORD addr, fBYTE value)
{
	addr &= 0x3FFF;
	if(addr < 0x2000) { if(!vrom)vpage[addr/0x400][addr&0x3FF] = value; return; }
	if(addr < 0x3F00)
	{
		addr &= 0x1FFF;
		unsigned ind = addr / 0x400;
		namtab[ind][addr & 0x3FF] = value;
	}
	addr &= 0x0F;
	palette[addr | 0x10] = palette[addr] = value & 0x3F;
}

BYTE bisqppu::readmem(fWORD addr) const
{
	addr &= 0x3FFF;
	if(addr < 0x2000) return vpage[addr/0x400][addr&0x3FF];
	if(addr < 0x3F00)
	{
		addr &= 0x1FFF;
		unsigned ind = addr / 0x400;
		return namtab[ind][addr & 0x3FF];
	}
	return palette[addr & 0x1F];
}

bool bisqppu::write(const fWORD addr, fBYTE value)
{
	switch(addr)
	{
		case 0x2000: reg2000 = value; return true;
		case 0x2001: reg2001 = value; return true;
		case 0x2003: reg2003 = value; return true;
		case 0x2004:
			spr_ram[reg2003++] = value;
			return true;
		case 0x2006:
			memptr = (memptr<<8) | value;
			return true;
		case 0x2007:
			writemem(memptr, value);
			memptr += (reg2000&4) ? 32 : 1;
			return true;
	}
	return false;
}

bool bisqppu::read(const fWORD addr, BYTE &value)
{
	switch(addr)
	{
		case 0x2002:
			if(vblank) { value |= 0x80; vblank = false; }
			return true;
		case 0x2007:
			value = readmem(memptr);
			memptr += (reg2000&4) ? 32 : 1;
			return true;
	}
	return false;
}

/* We do it *exactly* like NES, no matter how slow it is */
InterruptType bisqppu::tick()
{
	if(tickpos == 170*2)
	{
		tickpos = 0;
		++scanlid;
		if(scanlid == Clock->VisibleLines)
		{
			fprintf(stderr, "\33[0;31mvb\33[m");
			fflush(stderr);
			vblank = true;
		}
		else if(scanlid == Clock->ScanlineCount)
   		{
   			fflush(stderr);
   			scanlid = 0;
   			vblank = false;
   			if(reg2000 & 0x80)
   			{
   				fprintf(stderr, "\33[0;1;31mN\33[m");
				fflush(stderr);
   				return INT_NMI;
   			}
   		}
   		return INT_NONE;
	}
	if(!(tickpos&1))
	{
		if(tickpos < 256)
		{
			// Draw background 8 pix
		}
		else
		{
			// Fetch sprite for next scanline
		}
	}
	++tickpos;
	return INT_NONE;
}
