#ifndef BisqPPU_h
#define BisqPPU_h

#include <cstdio> /* For NULL */
using namespace std;

#include "cpu.hh"

class bisqppu
{
	friend class bisqnes;
	
    BYTE vram[0x2000];
    BYTE *vpage[8];    // 1k pages (0x400)
    BYTE *vrom;
    
    typedef BYTE natab[0x400];
    natab namtab[4];
    BYTE palette[0x20];
    
    BYTE spr_ram[256];
    
    BYTE nametable(unsigned ind, unsigned ofs) const { return namtab[ind][ofs]; }
    BYTE attrtable(unsigned ind, unsigned ofs) const { return namtab[ind][ofs|0x3C0]; }
    
    void writemem(fWORD addr, fBYTE value);
    BYTE readmem(fWORD addr) const;
	
public:
	void mapvram()
	{
		for(unsigned a=0; a<8; ++a)
			vpage[a] = &vram[a*0x400];
	}
	void resizevrom(unsigned size)
	{
		if(vrom)delete[] vrom;
		vrom = new BYTE[size];
		for(unsigned a=0; a<8; ++a)
			vpage[a] = &vrom[a*0x400];
	}
	BYTE *vromptr() { return &vrom[0]; }
	
public:
	unsigned dmaptr;
	WORD memptr;
	BYTE reg2000;
	BYTE reg2001;
	BYTE reg2003;
	bool vblank;
	
	unsigned tickpos, scanlid;
	bisqppu() : vrom(NULL)
	{
		dmaptr = 0;
		tickpos = 0;
		scanlid = 0;
		memptr = 0;
		reg2000 = 0x80;
		
		mapvram();
	}
	~bisqppu()
	{
		if(vrom)delete[] vrom;
	}

	bool write(const fWORD addr, fBYTE value);
	bool read(const fWORD addr, BYTE &value);
	
	InterruptType tick();
};

#endif
